/*
  Copyright (C) 2003 Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.aspects.gui.reports;

import dori.jasper.engine.JRDataSource;
import dori.jasper.engine.JRException;
import dori.jasper.engine.JRField;
import org.objectweb.jac.core.rtti.ClassItem;
import org.objectweb.jac.core.ObjectRepository;
import java.util.Iterator;

/**
 * A data source for JasperReports.
 */
public class JacDataSource implements JRDataSource {
   ClassItem cl;
   Iterator it;
   Object current;

   /**
    * Create a data source of all instances of a class.
    */
   public JacDataSource(ClassItem cl) {
      this.cl = cl;
   }

   // implementation of dori.jasper.engine.JRDataSource interface

   public boolean next() throws JRException
   {
      if (it==null) {
         it = ObjectRepository.getObjects(cl).iterator();
      }
      boolean result = it.hasNext();
      if (result)
         current = it.next();
      return result;
   }

   public Object getFieldValue(JRField field) throws JRException
   {
      if (current==null) {
         throw new JRException("No current "+cl.getName()+
                               " object to get field "+field.getName());
      } else {
         return cl.getField(field.getName()).getThroughAccessor(current);
      }
   }

}
