/*
  Copyright (C) 2001 Renaud Pawlak, Laurent Martelli
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.gui.swing;

import org.objectweb.jac.aspects.gui.ResourceManager;
import org.objectweb.jac.aspects.gui.FieldEditor;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.util.Log;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

/**
 * This is a special value editor that allows the user to nicely edit
 * an File. 
 */

public class FileEditor extends AbstractFieldEditor
   implements FieldEditor, ActionListener
{
   /** Stores the URL value. */
   protected JTextField textField;

   /**
    * Constructs a new URL editor.
    */

   public FileEditor(Object substance, FieldItem field) {
      super(substance,field);
      textField = new JTextField(10);
      textField.setMaximumSize(new Dimension(1000,22));
      textField.addFocusListener(this);
      add(textField);

      JButton button = new JButton(ResourceManager.getIconResource("edit_icon"));
      button.setToolTipText("Edit");
      button.setActionCommand("choose");
      button.addActionListener(this);
      add(button);
   }

   /**
    * Handles the actions performed by the users on this view.
    *
    * <p>On an URL editor, a "choose" action can be performed to allow
    * the user to open a file chooser box and to navigate within the
    * file system to choose his file.
    *
    * @param evt the performed action */

   public void actionPerformed(ActionEvent evt) {
      if (evt.getActionCommand().equals("choose")) {
         JFileChooser chooser = createFileChooser();
         int returnVal = chooser.showOpenDialog(this);
         
         if (returnVal==JFileChooser.APPROVE_OPTION) {
            textField.setText(chooser.getSelectedFile().toString());
         }
      }
   }

   /**
    * Returns a file chooser initialized with the current value
    */
   JFileChooser createFileChooser() {
      return new JFileChooser((File)getValue());
   }

   public Object getValue() {
      String file = textField.getText();
      if (file.equals("")) {
         return null;
      }
      return new File(file);
   }

   public void setValue(Object value) {
      super.setValue(value);
      if (value==null) 
         textField.setText("");
      else
         textField.setText(value.toString());
   }

   public void setWidth(int width) {
      textField.setColumns(width);
   }

   /**
    * Set the focus on the JTextField
    */
   public void requestFocus() {
      textField.requestFocus();
      Log.trace("gui.focus","focusing "+textField.getClass().getName());
   }

}
