/*
  Copyright (C) 2001-2003 Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.gui.web;

import org.objectweb.jac.aspects.gui.FieldEditor; 
import org.objectweb.jac.aspects.gui.FieldUpdate; 
import org.objectweb.jac.aspects.gui.FieldView; 
import org.objectweb.jac.aspects.gui.GuiAC; 
import org.objectweb.jac.aspects.gui.Utils;
import org.objectweb.jac.core.Collaboration;
import org.objectweb.jac.core.rtti.ClassItem;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.core.rtti.MethodItem;
import org.objectweb.jac.util.Enum;
import org.objectweb.jac.util.Log;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.lang.IllegalAccessException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

public abstract class AbstractFieldEditor extends AbstractView
   implements FieldEditor, FieldUpdate
{
   // substance and field are required so that we can register and
   // unregister ourself from fieldUpdated events on close()
   protected Object substance;
   protected FieldItem field;
   protected MethodItem setter;
   protected Object value;
   Object oldValue;
   protected boolean isEmbedded;
   boolean isValueSet = false;

   public AbstractFieldEditor(Object substance, FieldItem field) {
      this.substance = substance;
      setField(field);
   }

   public Object getValue() {
      return value;
   }

   public void setValue(Object value) {
      Log.trace("gui.editor",2,this+".setValue "+oldValue+"->"+value);
      this.value = value;
      if (!isValueSet) {
         oldValue = value;
         isValueSet = true;
      }
   }

   public void setSubstance(Object substance) {
      this.substance = substance;
   }

   public void setField(FieldItem field) {
      Utils.unregisterField(substance,this.field,this);
      this.field = field;
      if (field!=null)
         this.setter = field.getSetter();
      Utils.registerField(substance,field,this);
   }

   public FieldItem getField() {
      return field;
   }

   public void setAutoUpdate(boolean autoUpdate) {
   }

   public void setEmbedded(boolean isEmbedded) {
      this.isEmbedded = isEmbedded;
   }

   public void close() {
      if (Collaboration.get().getAttribute(GuiAC.NO_COMMIT)==null) {
         Object paramValue = 
            ((WebDisplay)context.getDisplay()).getRequest().getParameter(getLabel());
         Log.trace("gui.editor","reading value for "+label+": "+paramValue);
         if (paramValue!=null)
            readValue(paramValue);
         if (isEmbedded)
            commit();
      }
      Utils.unregisterField(substance,field,this);
   }

   public void onSetFocus(Object param) {
   }

   public boolean readValue(Object parameter) {
      if (disabled)
         return false;
      else 
         return doReadValue(parameter);
   }
   
   protected abstract boolean doReadValue(Object parameter);

   /**
    * Commit editing by calling the setter method.
    */
   public void commit() {
      if (Collaboration.get().getAttribute(GuiAC.NO_COMMIT)==null) {
         if (setter!=null && valueHasChanged()) {
            Log.trace("gui.editor",this+": "+field.getName()+
                      "'s value changed: "+getValue());
            try {
               field.setThroughWriter(substance,getValue());
            } catch (IllegalAccessException e) {
               Log.error("Failed to commit value "+getValue()+
                         " for field "+substance+"."+field);
               e.printStackTrace();
            }
            oldValue = this.value;
         }
      }
   }

   /**
    * Tells wether the value in the editor was changed
    */
   boolean valueHasChanged() {
      boolean ret;
      if( oldValue == null &&  value != null ) {
         ret = true;
      } else if( oldValue == null ) {
         ret = false;
      } else {
         ret = ! oldValue.equals(value);
      }
      Log.trace("gui.editor",2,"valueHasChanged("+field.getName()+") "+
                oldValue+" / "+value+" -> "+ret);
      return ret;
   }

   public void fieldUpdated(Object substance, FieldItem field, 
                            Object value, Object param) {
      setValue(value);
   }

   // When disabled == true, readValue() does nothing
   boolean disabled = false;
   public void disable() {
      Log.trace("gui.editor","disable "+this);
      disabled = true;
   }
   public void enable() {
      Log.trace("gui.editor","enable "+this);
      disabled = false;
   }
}
