/*
  Copyright (C) 2001-2003 Laurent Martelli <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.gui.web;

import org.objectweb.jac.util.Log;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.Date;
import org.mortbay.util.Resource;
 
public class ClasspathResource extends Resource {
   long lastModified = new Date().getTime();
   String path;
   File file;
   public ClasspathResource(String path) {
      if (path.startsWith("/")) {
         path = path.substring(1);
      }
      this.path = path;
      URL url = getClass().getClassLoader().getResource(path);
      if (url==null) {
         throw new RuntimeException("Resource not found: "+path);
      }
      file = new File(url.getFile());
   }
   public ClasspathResource() {
      path = null;
   }
   public void release() {
   }
   public boolean exists() {
      return getInputStream()!=null;
   }
   public boolean isDirectory() {
      return false;
   }
   public long lastModified() {
      if (file.exists())
         return file.lastModified();
      else
         return lastModified;
   }
   public long length() {
      try {
         InputStream is = getInputStream();
         if (is!=null)
            return getInputStream().available();
         else 
            return 0;
      } catch(Exception e) {
         Log.error("Failed to get length of resource: "+path);
         e.printStackTrace();
         return 0;
      }
   }
   public URL getURL() {
      return null;
   }
   public File getFile() {
      return file;
   }
   public String getName() {
      return path;
   }
   public InputStream getInputStream() {
      return getClass().getClassLoader().getResourceAsStream(path);
   }
   public OutputStream getOutputStream() {
      return null;
   }
   public boolean delete() {
      return false;
   }
   public boolean renameTo(Resource newName) {
      return false;
   }
   public String[] list() {
      return new String[0];
   }
   public Resource addPath(String addedPath) {
      if (path==null)
         return new ClasspathResource(addedPath);
      else 
         return this;
   }
   public String toString() {
      return path;
   }

}
