/*
  Copyright (C) 2001-2003 Laurent Martelli <laurent@aopsys.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.aspects.gui.web;

import org.objectweb.jac.aspects.gui.GuiAC;
import org.objectweb.jac.core.rtti.FieldItem;
import org.objectweb.jac.util.WrappedThrowableException;
import java.io.PrintWriter;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * HTML editor and viewer for dates.
 *
 * @see org.objectweb.jac.aspects.gui.GuiConf#setDateFormat(String)
 * @see java.util.Date
 */
public class DateEditor extends AbstractFieldEditor 
   implements HTMLEditor
{
   SimpleDateFormat format = new SimpleDateFormat(GuiAC.getDateFormat());

   public DateEditor(Object substance, FieldItem field) {
      super(substance,field);
   }

   // HTMLEditor interface

   public void genHTML(PrintWriter out) {
      out.print("<input type=\"text\" name=\""+label+"\" size=\"12\" style=\"width:12ex\" "+
                "value=\""+(value!=null?format.format((Date)value):"")+"\">");
   }

   protected boolean doReadValue(Object parameter) {
      String string = (String)parameter;
      try {
         Object date;
         if (string==null || string.length()==0)
            date = null;
         else
            date = format.parse(string);
         setValue(date);
         return true;
      } catch(ParseException e) {
         throw new WrappedThrowableException(e);
      }
   }


}

