/*
  Copyright (C) 2001-2003 Laurent Martelli <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.aspects.persistence;

import java.util.*;
import org.objectweb.jac.core.AspectComponent;
import org.objectweb.jac.core.Interaction;
import org.objectweb.jac.core.Wrappee;
import org.objectweb.jac.core.Wrapping;
import org.objectweb.jac.util.*;
import org.objectweb.jac.util.ExtBoolean;

/**
 * A wrapper for the Set interface.
 */
public class SetWrapper extends CollectionWrapper {

   public SetWrapper(AspectComponent ac, boolean isLoaded) {
      super(ac,isLoaded);
   }

   protected void doLoad(Wrappee wrappee) throws Exception {
      Log.trace("persistence","do load "+wrappee.getClass());
      // Keep the number of dynamic invocations to the minimum
      OID oid = getOID(wrappee);
      List list = getStorage().getList(oid);
      for(int i=0; i<list.size(); i++) {
         try {
            list.set(i,normalizeOutput(list.get(i)));
         } catch (NoSuchOIDError e) {
            Log.error("SetWrapper.doLoad("+oid+
                      "): skipping object with unknown OID "+list.get(i));
            list.set(i,null);
         }
      }
      attrdef(ATTR_ADDED,"true");
      Wrapping.invokeOrg(wrappee,"addAll",new Object[] { list } );
      attrdef(ATTR_ADDED,null);
   }

   public Object contains(Interaction interaction) throws Exception {
      if (isLoaded) {
         return proceed(interaction);
      } else {
         return ExtBoolean.valueOf(getStorage().setContains(
            getOID(interaction.wrappee),
            normalizeInput(interaction.args[0])));
      }
   }

   public boolean add(Interaction interaction) throws Exception {
      if (interaction.args[0]!=null)
         Log.trace("persistence","adding "+
                   interaction.args[0].getClass().getName()+" to set");
      if (isLoaded)
         proceed(interaction);
      return getStorage().addToSet(
         getOID(interaction.wrappee),normalizeInput(interaction.args[0]));
   }

   public Object remove(Interaction interaction) throws Exception {
      boolean result1 = false;
      if (isLoaded) {
         result1 = ((Boolean)proceed(interaction)).booleanValue();
      }
      boolean result2 = getStorage().removeFromSet(getOID(interaction.wrappee),
                                                   normalizeInput(interaction.args[0]));
      if (isLoaded) {
         if (result1!=result2)
            throw new Error("SetWrapper.remove result1 != result2");
         return ExtBoolean.valueOf(result1);
      } else {
         return ExtBoolean.valueOf(result2);
      }
   }

   /**
    * Remove all instances from the collection
    */
   public Object clear(Interaction interaction) throws Exception {
      Object result = proceed(interaction);
      getStorage().clearSet(getOID(interaction.wrappee));
      return result;
   }

   protected long getCollectionSize(OID oid) throws Exception {
      return getStorage().getSetSize(oid);
   }

   public Object iterator(Interaction interaction) {
      return new SetIterator(interaction.wrappee);
   }
}

