/*
  Copyright (C) 2002-2003 Laurent Martelli <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.ide;

import java.util.List;
import java.util.Map;
import java.util.Iterator;

import org.objectweb.jac.aspects.gui.HandlerResult;

import org.objectweb.jac.util.Log;

public class Errors {
   List errors;

   public Errors(List errors) {
      this.errors = errors;
   }

   public List getErrors() {
      return errors;
   }

   /*
    * GUI methods
    */

   
   public static HandlerResult selectionHandler(
      org.objectweb.jac.aspects.gui.DisplayContext context,
      org.objectweb.jac.core.rtti.CollectionItem collection,
      Error error)
   {
      Class cl = error.getCl();
      if (cl==null)
         return null;
      Map lineNumbers = CodeGeneration.getClassLineNumbers(cl);
      Member member = null;
      Iterator i = lineNumbers.entrySet().iterator();
      int closestLine = 0;
      while (i.hasNext()) {
         Map.Entry entry = (Map.Entry)i.next();
         int line = ((Integer)entry.getValue()).intValue();
         Log.trace("ide.compile",line+" -> "+entry.getKey());
         if (error.getLine() > line && line > closestLine) {
            closestLine = line;
            Log.trace("ide.compile","closestLine = "+closestLine);
            member = (Member)entry.getKey();
         }
      }
      if (member!=null)
         if (member instanceof Method) {
            Log.trace("ide.compile","Going to "+member+":"+(error.getLine()-closestLine-1));
            Method method = (Method)member;
            return new HandlerResult(
               context.getDisplay().getCustomizedView("ide"),
               org.objectweb.jac.core.rtti.ClassRepository.get()
                  .getClass(org.objectweb.jac.ide.Class.class).getField("methods"),
               method,"body",new Integer(error.getLine()-closestLine-1));
         } else {
            Log.trace("ide.compile","Going to "+member);
            return new HandlerResult(
               context.getDisplay().getCustomizedView("ide"),
               org.objectweb.jac.core.rtti.ClassRepository.get()
                  .getClass(org.objectweb.jac.ide.Class.class).getField("fields"),
               member,null,null);
         }
      else
         return null;
   }
}
