/*
  Copyright (C) 2002 Laurent Martelli <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

package org.objectweb.jac.ide;

// Look and feels

import org.objectweb.jac.aspects.gui.CustomizedDisplay;
import org.objectweb.jac.core.Collaboration;
import org.objectweb.jac.core.ObjectRepository;
import org.objectweb.jac.core.rtti.ClassRepository;
import org.objectweb.jac.util.Log;
import java.awt.Component;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 * Gui methods for the menus.
 */
public class Gui {

   protected static void setLookAndFeel(LookAndFeel look) 
      throws UnsupportedLookAndFeelException 
   {
      Object d = Collaboration.get().getAttribute("Gui.display");
      if (d instanceof CustomizedDisplay) {
         CustomizedDisplay display = (CustomizedDisplay)d;
         Log.trace("ide","setLookAndFeel "+look.getClass().getName());
         UIManager.setLookAndFeel(look);
         Iterator i = display.getCustomizedViews().iterator();
         while (i.hasNext()) {
            Object frame = i.next();
            if (frame instanceof Component) {
               Log.trace("ide","updateComponentTreeUI on "+frame);
               SwingUtilities.updateComponentTreeUI((Component)frame);
            }
         }
      }
      
   }

   /*
   public static void setKunstoffLookAndFeel() 
      throws UnsupportedLookAndFeelException 
   {
      setLookAndFeel(new com.incors.plaf.kunststoff.KunststoffLookAndFeel());
   }
   */

   /**
    * Returns all types which belong to the same project as a
    * given member.
    * @param substance 
    */
   public static Collection getAvailableTypes(ModelElement substance) {
      Vector result = new Vector();
      Project project = null;
      if (substance instanceof Member) {
         project = ((Member)substance).getProject();
      } else if (substance instanceof Parameter) {
         Method method = ((Parameter)substance).getMethod();
         if (method!=null)
            project = method.getProject();
      }
      Collection types = ObjectRepository.getObjects(
         ClassRepository.get().getClass("org.objectweb.jac.ide.Type"));
      Iterator it = types.iterator();
      while (it.hasNext()) {
         Type type = (Type)it.next();
         if (!(type instanceof Class && project!=null &&
               ((Class)type).getProject()!=project)) {
            result.add(type);
         }
      }
      return result;
   }
}

