/*
  Copyright (C) 2002-2003 Laurent Martelli <laurent@aopsys.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

package org.objectweb.jac.util;

/**
 * Various often used array functions
 */
public class ExtArrays
{
   /**
    * Insert an object into an array
    * @param position of the object to insert (>=0 and <=array.length)
    * @param toInsert the object to insert
    * @param array the array
    * @return a new array of size array.length+1, where item at
    * position "position" is toInsert 
    */
   public static Object[] add(int position, Object toInsert, Object[] array) {
      Object[] newArray = new Object[array.length+1];
      if (position>0)
         System.arraycopy(array,0,newArray,0,position);
      newArray[position] = toInsert;
      if (position<array.length) {
         System.arraycopy(array,position,newArray,position+1,array.length-position);
      }
      return newArray;
   }

   /**
    * Append an object at the end of an array
    * @param toAppend the object to add
    * @param array the array
    * @return a new array, of length array.length+1 and whose last item is toAppend
    */
   public static Object[] add(Object toAppend, Object[] array) {
      return add(array.length,toAppend,array);
   }

   /**
    * Returns the index of a value in an array of Object.
    *
    * @param array the array
    * @param value the searched value
    * @return Returns the lowest integer value i such that array[i]==value, or -1.
    */
   public static int indexOf(Object[] array, Object value) {
      for (int i=0; i<array.length; i++) {
         if (value==array[i]) {
            return i;
         }
      }
      return -1;
   }

   /**
    * Tells wether an array of objects contains a given value
    * @param array the array to search the value in
    * @param value the object to search for
    * @return true if array contains value
    */
   public static boolean contains(Object[] array, Object value) {
      return indexOf(array,value)!=-1;
   }
}
